int relays[]={A0,A1,A2,A3};    //button pins
int outstates[]={0,0,0,0};      //initial output state

#define LORAFREQ (915000000L)
#include <SPI.h>
#include <LoRa.h>
unsigned long tmout;

void setup() {
  for(int i=0;i<4;i++){
    pinMode(relays[i],OUTPUT);            //output
    digitalWrite(relays[i],LOW);          //off
  }
  LoRa.begin(LORAFREQ);
  LoRa.onReceive(onReceive);
  LoRa.receive();
  tmout=millis();

}

void loop() {
  if(millis()-tmout>9000){               //send every 10 seconds anyway
    for(int i=0;i<4;i++){
      sendpacket(i);
      delay(5);                          //space out transmissions
    }
    tmout=millis();
  }   
}

void onReceive(int packetSize) {
  char buf[20]="";  
  int k;
  int n;
  for (int i = 0; i < packetSize; i++) {
    k=i;
    if(k>18){k=18;}                         //make sure we don't write past end of string
    buf[k]=(char)LoRa.read();
  }
  if(buf[0]!='B'){return;}        //jump out if wrong data
  if(buf[1]!='U'){return;}        //jump out if wrong data
  if(buf[2]!='T'){return;}        //jump out if wrong data
  if(buf[3]!='T'){return;}        //jump out if wrong data
  if(buf[4]!='O'){return;}        //jump out if wrong data
  if(buf[5]!='N'){return;}        //jump out if wrong data
  if(buf[7]!='O'){return;}        //jump out if wrong data
  n=buf[6]-'1';
  if((n<0)||(n>3)){return;}       //invalid number
  if((buf[8]=='F')&&(buf[9]=='F')){outstates[n]=0;}    //off received
  if((buf[8]=='N')){outstates[n]=1;}    //on received
  digitalWrite(relays[n],outstates[n]);
  sendpacket(n);
}

void sendpacket(int i){       //send current state of i
  LoRa.beginPacket();
  LoRa.print("RELAYS");
  LoRa.print(i+1);
  if(outstates[i]){
    LoRa.print("ON");
  }else{
    LoRa.print("OFF");
  }
  LoRa.endPacket();
  LoRa.receive();               
}



