int buttons[]={A0,A1,A2,A3};    //button pins
int leds[]={7,6,5,4};           //LED pins- bicolour
int outstates[]={0,0,0,0};      //initial output state
int instates[]={-1,-1,-1,-1};   //initial input state (invalid)
int lastbutton[]={1,1,1,1};     //last button state

#define LORAFREQ (915000000L)
#include <SPI.h>
#include <LoRa.h>
unsigned long tmout;

void setup() {    
  for(int i=0;i<4;i++){
    pinMode(buttons[i],INPUT_PULLUP);   //buttons short to ground when pressed
    pinMode(leds[i],OUTPUT);            //on
    digitalWrite(leds[i],LOW);          //red
  }
  LoRa.begin(LORAFREQ);
  LoRa.onReceive(onReceive);
  LoRa.receive();
  tmout=millis();
}

void loop() {
  for(int i=0;i<4;i++){
    int d=digitalRead(buttons[i]);
    if((d==0)&&(lastbutton[i])){          //button pressed
      delay(5);                           //basic debounce
      if(digitalRead(buttons[i])==0){     //stable
        outstates[i]=1-outstates[i];      //toggle
        sendpacket(i);
      }
    }
    lastbutton[i]=d;
    digitalWrite(leds[i],outstates[i]);   //change LED
    pinMode(leds[i],OUTPUT);
    if((instates[i]!=outstates[i])&&(((millis()/100)%2)==0)){pinMode(leds[i],INPUT);}   //flash LED off if invalid feedback
  }
  if(millis()-tmout>10000){               //send every 10 seconds anyway
    for(int i=0;i<4;i++){
      sendpacket(i);
      delay(50);                          //space out transmissions
    }
    tmout=millis();
  }   
}

void sendpacket(int i){       //send current state of i
  LoRa.beginPacket();
  LoRa.print("BUTTON");
  LoRa.print(i+1);
  if(outstates[i]){
    LoRa.print("ON");
  }else{
    LoRa.print("OFF");
  }
  LoRa.endPacket();
  LoRa.receive();               
}

void onReceive(int packetSize) {
  char buf[20]="";  
  int k;
  int n;
  for (int i = 0; i < packetSize; i++) {
    k=i;
    if(k>18){k=18;}                         //make sure we don't write past end of string
    buf[k]=(char)LoRa.read();
  }
  if(buf[0]!='R'){return;}        //jump out if wrong data
  if(buf[1]!='E'){return;}        //jump out if wrong data
  if(buf[2]!='L'){return;}        //jump out if wrong data
  if(buf[3]!='A'){return;}        //jump out if wrong data
  if(buf[4]!='Y'){return;}        //jump out if wrong data
  if(buf[5]!='S'){return;}        //jump out if wrong data
  if(buf[7]!='O'){return;}        //jump out if wrong data
  n=buf[6]-'1';
  if((n<0)||(n>3)){return;}       //invalid number
  if((buf[8]=='F')&&(buf[9]=='F')){instates[n]=0;}    //off received
  if((buf[8]=='N')){instates[n]=1;}    //on received
}


